==> robomaker-2.13.12-sources.jar: software/amazon/awssdk/services/robomaker/paginators/ListSimulationJobBatchesPublisher.java <==
/*
 * Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You may not use this file except in compliance with
 * the License. A copy of the License is located at
 * 
 * http://aws.amazon.com/apache2.0
 * 
 * or in the "license" file accompanying this file. This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
 * CONDITIONS OF ANY KIND, either express or implied. See the License for the specific language governing permissions
 * and limitations under the License.
 */

package software.amazon.awssdk.services.robomaker.paginators;

import java.util.Collections;
import java.util.Iterator;
import java.util.concurrent.CompletableFuture;
import java.util.function.Function;
import org.reactivestreams.Subscriber;
import software.amazon.awssdk.annotations.Generated;
import software.amazon.awssdk.core.async.SdkPublisher;
import software.amazon.awssdk.core.pagination.async.AsyncPageFetcher;
import software.amazon.awssdk.core.pagination.async.PaginatedItemsPublisher;
import software.amazon.awssdk.core.pagination.async.ResponsesSubscription;
import software.amazon.awssdk.core.util.PaginatorUtils;
import software.amazon.awssdk.services.robomaker.RoboMakerAsyncClient;
import software.amazon.awssdk.services.robomaker.model.ListSimulationJobBatchesRequest;
import software.amazon.awssdk.services.robomaker.model.ListSimulationJobBatchesResponse;
import software.amazon.awssdk.services.robomaker.model.SimulationJobBatchSummary;

/**
 * <p>
 * Represents the output for the
 * {@link software.amazon.awssdk.services.robomaker.RoboMakerAsyncClient#listSimulationJobBatchesPaginator(software.amazon.awssdk.services.robomaker.model.ListSimulationJobBatchesRequest)}
 * operation which is a paginated operation. This class is a type of {@link org.reactivestreams.Publisher} which can be
 * used to provide a sequence of
 * {@link software.amazon.awssdk.services.robomaker.model.ListSimulationJobBatchesResponse} response pages as per demand
 * from the subscriber.
 * </p>
 * <p>
 * When the operation is called, an instance of this class is returned. At this point, no service calls are made yet and
 * so there is no guarantee that the request is valid. If there are errors in your request, you will see the failures
 * only after you start streaming the data. The subscribe method should be called as a request to start streaming data.
 * For more info, see {@link org.reactivestreams.Publisher#subscribe(org.reactivestreams.Subscriber)}. Each call to the
 * subscribe method will result in a new {@link org.reactivestreams.Subscription} i.e., a new contract to stream data
 * from the starting request.
 * </p>
 *
 * <p>
 * The following are few ways to use the response class:
 * </p>
 * 1) Using the subscribe helper method
 * 
 * <pre>
 * {@code
 * software.amazon.awssdk.services.robomaker.paginators.ListSimulationJobBatchesPublisher publisher = client.listSimulationJobBatchesPaginator(request);
 * CompletableFuture<Void> future = publisher.subscribe(res -> { // Do something with the response });
 * future.get();
 * }
 * </pre>
 *
 * 2) Using a custom subscriber
 * 
 * <pre>
 * {@code
 * software.amazon.awssdk.services.robomaker.paginators.ListSimulationJobBatchesPublisher publisher = client.listSimulationJobBatchesPaginator(request);
 * publisher.subscribe(new Subscriber<software.amazon.awssdk.services.robomaker.model.ListSimulationJobBatchesResponse>() {
 * 
 * public void onSubscribe(org.reactivestreams.Subscriber subscription) { //... };
 * 
 * 
 * public void onNext(software.amazon.awssdk.services.robomaker.model.ListSimulationJobBatchesResponse response) { //... };
 * });}
 * </pre>
 * 
 * As the response is a publisher, it can work well with third party reactive streams implementations like RxJava2.
 * <p>
 * <b>Note: If you prefer to have control on service calls, use the
 * {@link #listSimulationJobBatches(software.amazon.awssdk.services.robomaker.model.ListSimulationJobBatchesRequest)}
 * operation.</b>
 * </p>
 */
@Generated("software.amazon.awssdk:codegen")
public class ListSimulationJobBatchesPublisher implements SdkPublisher<ListSimulationJobBatchesResponse> {
    private final RoboMakerAsyncClient client;

    private final ListSimulationJobBatchesRequest firstRequest;

    private final AsyncPageFetcher nextPageFetcher;

    private boolean isLastPage;

    public ListSimulationJobBatchesPublisher(RoboMakerAsyncClient client, ListSimulationJobBatchesRequest firstRequest) {
        this(client, firstRequest, false);
    }

    private ListSimulationJobBatchesPublisher(RoboMakerAsyncClient client, ListSimulationJobBatchesRequest firstRequest,
            boolean isLastPage) {
        this.client = client;
        this.firstRequest = firstRequest;
        this.isLastPage = isLastPage;
        this.nextPageFetcher = new ListSimulationJobBatchesResponseFetcher();
    }

    @Override
    public void subscribe(Subscriber<? super ListSimulationJobBatchesResponse> subscriber) {
        subscriber.onSubscribe(ResponsesSubscription.builder().subscriber(subscriber).nextPageFetcher(nextPageFetcher).build());
    }

    /**
     * Returns a publisher that can be used to get a stream of data. You need to subscribe to the publisher to request
     * the stream of data. The publisher has a helper forEach method that takes in a {@link java.util.function.Consumer}
     * and then applies that consumer to each response returned by the service.
     */
    public final SdkPublisher<SimulationJobBatchSummary> simulationJobBatchSummaries() {
        Function<ListSimulationJobBatchesResponse, Iterator<SimulationJobBatchSummary>> getIterator = response -> {
            if (response != null && response.simulationJobBatchSummaries() != null) {
                return response.simulationJobBatchSummaries().iterator();
            }
            return Collections.emptyIterator();
        };
        return PaginatedItemsPublisher.builder().nextPageFetcher(new ListSimulationJobBatchesResponseFetcher())
                .iteratorFunction(getIterator).isLastPage(isLastPage).build();
    }

    private class ListSimulationJobBatchesResponseFetcher implements AsyncPageFetcher<ListSimulationJobBatchesResponse> {
        @Override
        public boolean hasNextPage(final ListSimulationJobBatchesResponse previousPage) {
            return PaginatorUtils.isOutputTokenAvailable(previousPage.nextToken());
        }

        @Override
        public CompletableFuture<ListSimulationJobBatchesResponse> nextPage(final ListSimulationJobBatchesResponse previousPage) {
            if (previousPage == null) {
                return client.listSimulationJobBatches(firstRequest);
            }
            return client.listSimulationJobBatches(firstRequest.toBuilder().nextToken(previousPage.nextToken()).build());
        }
    }
}

==> robomaker-2.20.43-sources.jar: software/amazon/awssdk/services/robomaker/paginators/ListSimulationJobBatchesPublisher.java <==
/*
 * Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"). You may not use this file except in compliance with
 * the License. A copy of the License is located at
 * 
 * http://aws.amazon.com/apache2.0
 * 
 * or in the "license" file accompanying this file. This file is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR
 * CONDITIONS OF ANY KIND, either express or implied. See the License for the specific language governing permissions
 * and limitations under the License.
 */

package software.amazon.awssdk.services.robomaker.paginators;

import java.util.Collections;
import java.util.Iterator;
import java.util.concurrent.CompletableFuture;
import java.util.function.Function;
import org.reactivestreams.Subscriber;
import software.amazon.awssdk.annotations.Generated;
import software.amazon.awssdk.core.async.SdkPublisher;
import software.amazon.awssdk.core.pagination.async.AsyncPageFetcher;
import software.amazon.awssdk.core.pagination.async.PaginatedItemsPublisher;
import software.amazon.awssdk.core.pagination.async.ResponsesSubscription;
import software.amazon.awssdk.core.util.PaginatorUtils;
import software.amazon.awssdk.services.robomaker.RoboMakerAsyncClient;
import software.amazon.awssdk.services.robomaker.model.ListSimulationJobBatchesRequest;
import software.amazon.awssdk.services.robomaker.model.ListSimulationJobBatchesResponse;
import software.amazon.awssdk.services.robomaker.model.SimulationJobBatchSummary;

/**
 * <p>
 * Represents the output for the
 * {@link software.amazon.awssdk.services.robomaker.RoboMakerAsyncClient#listSimulationJobBatchesPaginator(software.amazon.awssdk.services.robomaker.model.ListSimulationJobBatchesRequest)}
 * operation which is a paginated operation. This class is a type of {@link org.reactivestreams.Publisher} which can be
 * used to provide a sequence of
 * {@link software.amazon.awssdk.services.robomaker.model.ListSimulationJobBatchesResponse} response pages as per demand
 * from the subscriber.
 * </p>
 * <p>
 * When the operation is called, an instance of this class is returned. At this point, no service calls are made yet and
 * so there is no guarantee that the request is valid. If there are errors in your request, you will see the failures
 * only after you start streaming the data. The subscribe method should be called as a request to start streaming data.
 * For more info, see {@link org.reactivestreams.Publisher#subscribe(org.reactivestreams.Subscriber)}. Each call to the
 * subscribe method will result in a new {@link org.reactivestreams.Subscription} i.e., a new contract to stream data
 * from the starting request.
 * </p>
 *
 * <p>
 * The following are few ways to use the response class:
 * </p>
 * 1) Using the subscribe helper method
 * 
 * <pre>
 * {@code
 * software.amazon.awssdk.services.robomaker.paginators.ListSimulationJobBatchesPublisher publisher = client.listSimulationJobBatchesPaginator(request);
 * CompletableFuture<Void> future = publisher.subscribe(res -> { // Do something with the response });
 * future.get();
 * }
 * </pre>
 *
 * 2) Using a custom subscriber
 * 
 * <pre>
 * {@code
 * software.amazon.awssdk.services.robomaker.paginators.ListSimulationJobBatchesPublisher publisher = client.listSimulationJobBatchesPaginator(request);
 * publisher.subscribe(new Subscriber<software.amazon.awssdk.services.robomaker.model.ListSimulationJobBatchesResponse>() {
 * 
 * public void onSubscribe(org.reactivestreams.Subscriber subscription) { //... };
 * 
 * 
 * public void onNext(software.amazon.awssdk.services.robomaker.model.ListSimulationJobBatchesResponse response) { //... };
 * });}
 * </pre>
 * 
 * As the response is a publisher, it can work well with third party reactive streams implementations like RxJava2.
 * <p>
 * <b>Please notice that the configuration of maxResults won't limit the number of results you get with the paginator.
 * It only limits the number of results in each page.</b>
 * </p>
 * <p>
 * <b>Note: If you prefer to have control on service calls, use the
 * {@link #listSimulationJobBatches(software.amazon.awssdk.services.robomaker.model.ListSimulationJobBatchesRequest)}
 * operation.</b>
 * </p>
 */
@Generated("software.amazon.awssdk:codegen")
public class ListSimulationJobBatchesPublisher implements SdkPublisher<ListSimulationJobBatchesResponse> {
    private final RoboMakerAsyncClient client;

    private final ListSimulationJobBatchesRequest firstRequest;

    private final AsyncPageFetcher nextPageFetcher;

    private boolean isLastPage;

    public ListSimulationJobBatchesPublisher(RoboMakerAsyncClient client, ListSimulationJobBatchesRequest firstRequest) {
        this(client, firstRequest, false);
    }

    private ListSimulationJobBatchesPublisher(RoboMakerAsyncClient client, ListSimulationJobBatchesRequest firstRequest,
            boolean isLastPage) {
        this.client = client;
        this.firstRequest = firstRequest;
        this.isLastPage = isLastPage;
        this.nextPageFetcher = new ListSimulationJobBatchesResponseFetcher();
    }

    @Override
    public void subscribe(Subscriber<? super ListSimulationJobBatchesResponse> subscriber) {
        subscriber.onSubscribe(ResponsesSubscription.builder().subscriber(subscriber).nextPageFetcher(nextPageFetcher).build());
    }

    /**
     * Returns a publisher that can be used to get a stream of data. You need to subscribe to the publisher to request
     * the stream of data. The publisher has a helper forEach method that takes in a {@link java.util.function.Consumer}
     * and then applies that consumer to each response returned by the service.
     */
    public final SdkPublisher<SimulationJobBatchSummary> simulationJobBatchSummaries() {
        Function<ListSimulationJobBatchesResponse, Iterator<SimulationJobBatchSummary>> getIterator = response -> {
            if (response != null && response.simulationJobBatchSummaries() != null) {
                return response.simulationJobBatchSummaries().iterator();
            }
            return Collections.emptyIterator();
        };
        return PaginatedItemsPublisher.builder().nextPageFetcher(new ListSimulationJobBatchesResponseFetcher())
                .iteratorFunction(getIterator).isLastPage(isLastPage).build();
    }

    private class ListSimulationJobBatchesResponseFetcher implements AsyncPageFetcher<ListSimulationJobBatchesResponse> {
        @Override
        public boolean hasNextPage(final ListSimulationJobBatchesResponse previousPage) {
            return PaginatorUtils.isOutputTokenAvailable(previousPage.nextToken());
        }

        @Override
        public CompletableFuture<ListSimulationJobBatchesResponse> nextPage(final ListSimulationJobBatchesResponse previousPage) {
            if (previousPage == null) {
                return client.listSimulationJobBatches(firstRequest);
            }
            return client.listSimulationJobBatches(firstRequest.toBuilder().nextToken(previousPage.nextToken()).build());
        }
    }
}
